
package org.mule.modules.workday.staffing.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.workday.staffing.config.GetPositionsMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class GetPositionsDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public GetPositionsDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(GetPositionsMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        Element criteriaListElement = null;
        criteriaListElement = DomUtils.getChildElementByTagName(element, "criteria");
        List<Element> criteriaListChilds = null;
        if (criteriaListElement!= null) {
            String criteriaRef = criteriaListElement.getAttribute("ref");
            if ((criteriaRef!= null)&&(!StringUtils.isBlank(criteriaRef))) {
                if ((!criteriaRef.startsWith(patternInfo.getPrefix()))&&(!criteriaRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("criteria", new RuntimeBeanReference(criteriaRef));
                } else {
                    builder.addPropertyValue("criteria", criteriaRef);
                }
            } else {
                ManagedMap criteria = new ManagedMap();
                criteriaListChilds = DomUtils.getChildElementsByTagName(criteriaListElement, "criterium");
                if (criteriaListChilds!= null) {
                    if (criteriaListChilds.size() == 0) {
                        criteriaListChilds = DomUtils.getChildElements(criteriaListElement);
                    }
                    for (Element criteriaChild: criteriaListChilds) {
                        String criteriaValueRef = criteriaChild.getAttribute("value-ref");
                        String criteriaKeyRef = criteriaChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((criteriaValueRef!= null)&&(!StringUtils.isBlank(criteriaValueRef))) {
                            valueObject = new RuntimeBeanReference(criteriaValueRef);
                        } else {
                            valueObject = criteriaChild.getTextContent();
                        }
                        if ((criteriaKeyRef!= null)&&(!StringUtils.isBlank(criteriaKeyRef))) {
                            keyObject = new RuntimeBeanReference(criteriaKeyRef);
                        } else {
                            keyObject = criteriaChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = criteriaChild.getTagName();
                        }
                        criteria.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("criteria", criteria);
            }
        }
        Element referencesListElement = null;
        referencesListElement = DomUtils.getChildElementByTagName(element, "references");
        List<Element> referencesListChilds = null;
        if (referencesListElement!= null) {
            String referencesRef = referencesListElement.getAttribute("ref");
            if ((referencesRef!= null)&&(!StringUtils.isBlank(referencesRef))) {
                if ((!referencesRef.startsWith(patternInfo.getPrefix()))&&(!referencesRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("references", new RuntimeBeanReference(referencesRef));
                } else {
                    builder.addPropertyValue("references", referencesRef);
                }
            } else {
                ManagedList references = new ManagedList();
                referencesListChilds = DomUtils.getChildElementsByTagName(referencesListElement, "reference");
                if (referencesListChilds!= null) {
                    for (Element referencesChild: referencesListChilds) {
                        String valueRef = referencesChild.getAttribute("value-ref");
                        if ((valueRef!= null)&&(!StringUtils.isBlank(valueRef))) {
                            references.add(new RuntimeBeanReference(valueRef));
                        } else {
                            List<Element> innerreferenceListChilds = null;
                            if (referencesChild!= null) {
                                String innerreferenceRef = referencesChild.getAttribute("ref");
                                if ((innerreferenceRef!= null)&&(!StringUtils.isBlank(innerreferenceRef))) {
                                    if ((!innerreferenceRef.startsWith(patternInfo.getPrefix()))&&(!innerreferenceRef.endsWith(patternInfo.getSuffix()))) {
                                        builder.addPropertyValue("inner-reference", new RuntimeBeanReference(innerreferenceRef));
                                    } else {
                                        builder.addPropertyValue("inner-reference", innerreferenceRef);
                                    }
                                } else {
                                    ManagedMap innerreference = new ManagedMap();
                                    innerreferenceListChilds = DomUtils.getChildElementsByTagName(referencesChild, "inner-reference");
                                    if (innerreferenceListChilds!= null) {
                                        if (innerreferenceListChilds.size() == 0) {
                                            innerreferenceListChilds = DomUtils.getChildElements(referencesChild);
                                        }
                                        for (Element innerreferenceChild: innerreferenceListChilds) {
                                            String innerreferenceValueRef = innerreferenceChild.getAttribute("value-ref");
                                            String innerreferenceKeyRef = innerreferenceChild.getAttribute("key-ref");
                                            Object valueObject = null;
                                            Object keyObject = null;
                                            if ((innerreferenceValueRef!= null)&&(!StringUtils.isBlank(innerreferenceValueRef))) {
                                                valueObject = new RuntimeBeanReference(innerreferenceValueRef);
                                            } else {
                                                valueObject = innerreferenceChild.getTextContent();
                                            }
                                            if ((innerreferenceKeyRef!= null)&&(!StringUtils.isBlank(innerreferenceKeyRef))) {
                                                keyObject = new RuntimeBeanReference(innerreferenceKeyRef);
                                            } else {
                                                keyObject = innerreferenceChild.getAttribute("key");
                                            }
                                            if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                                                keyObject = innerreferenceChild.getTagName();
                                            }
                                            innerreference.put(keyObject, valueObject);
                                        }
                                    }
                                    references.add(innerreference);
                                }
                            }
                        }
                    }
                }
                builder.addPropertyValue("references", references);
            }
        }
        Element responseFilterListElement = null;
        responseFilterListElement = DomUtils.getChildElementByTagName(element, "response-filter");
        List<Element> responseFilterListChilds = null;
        if (responseFilterListElement!= null) {
            String responseFilterRef = responseFilterListElement.getAttribute("ref");
            if ((responseFilterRef!= null)&&(!StringUtils.isBlank(responseFilterRef))) {
                if ((!responseFilterRef.startsWith(patternInfo.getPrefix()))&&(!responseFilterRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("responseFilter", new RuntimeBeanReference(responseFilterRef));
                } else {
                    builder.addPropertyValue("responseFilter", responseFilterRef);
                }
            } else {
                ManagedMap responseFilter = new ManagedMap();
                responseFilterListChilds = DomUtils.getChildElementsByTagName(responseFilterListElement, "response-filter");
                if (responseFilterListChilds!= null) {
                    if (responseFilterListChilds.size() == 0) {
                        responseFilterListChilds = DomUtils.getChildElements(responseFilterListElement);
                    }
                    for (Element responseFilterChild: responseFilterListChilds) {
                        String responseFilterValueRef = responseFilterChild.getAttribute("value-ref");
                        String responseFilterKeyRef = responseFilterChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((responseFilterValueRef!= null)&&(!StringUtils.isBlank(responseFilterValueRef))) {
                            valueObject = new RuntimeBeanReference(responseFilterValueRef);
                        } else {
                            valueObject = responseFilterChild.getTextContent();
                        }
                        if ((responseFilterKeyRef!= null)&&(!StringUtils.isBlank(responseFilterKeyRef))) {
                            keyObject = new RuntimeBeanReference(responseFilterKeyRef);
                        } else {
                            keyObject = responseFilterChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = responseFilterChild.getTagName();
                        }
                        responseFilter.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("responseFilter", responseFilter);
            }
        }
        Element responseGroupListElement = null;
        responseGroupListElement = DomUtils.getChildElementByTagName(element, "response-group");
        List<Element> responseGroupListChilds = null;
        if (responseGroupListElement!= null) {
            String responseGroupRef = responseGroupListElement.getAttribute("ref");
            if ((responseGroupRef!= null)&&(!StringUtils.isBlank(responseGroupRef))) {
                if ((!responseGroupRef.startsWith(patternInfo.getPrefix()))&&(!responseGroupRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("responseGroup", new RuntimeBeanReference(responseGroupRef));
                } else {
                    builder.addPropertyValue("responseGroup", responseGroupRef);
                }
            } else {
                ManagedMap responseGroup = new ManagedMap();
                responseGroupListChilds = DomUtils.getChildElementsByTagName(responseGroupListElement, "response-group");
                if (responseGroupListChilds!= null) {
                    if (responseGroupListChilds.size() == 0) {
                        responseGroupListChilds = DomUtils.getChildElements(responseGroupListElement);
                    }
                    for (Element responseGroupChild: responseGroupListChilds) {
                        String responseGroupValueRef = responseGroupChild.getAttribute("value-ref");
                        String responseGroupKeyRef = responseGroupChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((responseGroupValueRef!= null)&&(!StringUtils.isBlank(responseGroupValueRef))) {
                            valueObject = new RuntimeBeanReference(responseGroupValueRef);
                        } else {
                            valueObject = responseGroupChild.getTextContent();
                        }
                        if ((responseGroupKeyRef!= null)&&(!StringUtils.isBlank(responseGroupKeyRef))) {
                            keyObject = new RuntimeBeanReference(responseGroupKeyRef);
                        } else {
                            keyObject = responseGroupChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = responseGroupChild.getTagName();
                        }
                        responseGroup.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("responseGroup", responseGroup);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
