
package org.mule.modules.workday.staffing.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.workday.staffing.config.PutApplicantMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class PutApplicantDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public PutApplicantDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(PutApplicantMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("applicantId")!= null)&&(!StringUtils.isBlank(element.getAttribute("applicantId")))) {
            builder.addPropertyValue("applicantId", element.getAttribute("applicantId"));
        }
        if ((element.getAttribute("workdayId")!= null)&&(!StringUtils.isBlank(element.getAttribute("workdayId")))) {
            builder.addPropertyValue("workdayId", element.getAttribute("workdayId"));
        }
        Element applicantListElement = null;
        applicantListElement = DomUtils.getChildElementByTagName(element, "applicant");
        List<Element> applicantListChilds = null;
        if (applicantListElement!= null) {
            String applicantRef = applicantListElement.getAttribute("ref");
            if ((applicantRef!= null)&&(!StringUtils.isBlank(applicantRef))) {
                if ((!applicantRef.startsWith(patternInfo.getPrefix()))&&(!applicantRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("applicant", new RuntimeBeanReference(applicantRef));
                } else {
                    builder.addPropertyValue("applicant", applicantRef);
                }
            } else {
                ManagedMap applicant = new ManagedMap();
                applicantListChilds = DomUtils.getChildElementsByTagName(applicantListElement, "applicant");
                if (applicantListChilds!= null) {
                    if (applicantListChilds.size() == 0) {
                        applicantListChilds = DomUtils.getChildElements(applicantListElement);
                    }
                    for (Element applicantChild: applicantListChilds) {
                        String applicantValueRef = applicantChild.getAttribute("value-ref");
                        String applicantKeyRef = applicantChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((applicantValueRef!= null)&&(!StringUtils.isBlank(applicantValueRef))) {
                            valueObject = new RuntimeBeanReference(applicantValueRef);
                        } else {
                            valueObject = applicantChild.getTextContent();
                        }
                        if ((applicantKeyRef!= null)&&(!StringUtils.isBlank(applicantKeyRef))) {
                            keyObject = new RuntimeBeanReference(applicantKeyRef);
                        } else {
                            keyObject = applicantChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = applicantChild.getTagName();
                        }
                        applicant.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("applicant", applicant);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
